/*
 * This program operates a device that can detect an alarm signal and depress a lever when that signal is detected.
 * The alarm signal can be custom recorded to the device. The function for that is record().
 * The function butPressed() handles button presses.
 * The function getSound() converts raw data from the microphone to a frequency characteristic of the recorded sound.
 * This frequency characteristic is the parameter used to compare sounds and recognize the alarm.
 */
 
#include <Servo.h>

int curVol; //Stores retrieved value from microphone
int pinBut = 12; //pin input from button
int pinG = 2; //pin output for green led
int pinR = 6; //pin output for red led
int pinVol = A0; //pin input from microphone
int pinServo = 3; //pin output for servo
int alarmSound; //stores frequency characteristic of alarm
int curSound; //stores current measured frequency characteristic
int fht_input[256]; //stores list of measured microphone inputs
Servo servo; //Servo motor initialization

void setup() {
  pinMode(pinVol,INPUT); //Set input for microphone
  pinMode(pinBut,INPUT); //Set input for button
  pinMode(pinR,OUTPUT); //Set output for red led
  pinMode(pinG,OUTPUT); //Set output for green led
  servo.attach(pinServo); //attach servo object to servo pin
  servo.write(180); //default to servo lever up
}

void loop() {
  if(digitalRead(pinBut)==HIGH){ //If button is pressed, run butPressed()
    butPressed();
  }
  curSound = getSound(); //retrieve frequency characteristic for currently heard sound
  if(curSound-alarmSound<50 && curSound-alarmSound>-50){ //if current freq. characteristic is similar to alarm
    servo.write(10); //servo lever to down
    delay(500);
    servo.write(180); //servo lever back up
    delay(500);
  }
}

void record() {//This function records and saves the frequency characteristic of the alarm
  delay(1000);
  digitalWrite(pinG,HIGH);//Indicate recording beginning with green led
  delay(500);
  alarmSound = getSound();//Record alarm
  delay(2000);
  digitalWrite(pinG,LOW);//Indicate recording ending with green led off
  digitalWrite(pinR,HIGH);//Indicate recording complete with red led
  delay(200);
  digitalWrite(pinR,LOW);//Indicate return to normal device operation with red led off
  delay(100);
}

int getSound() {//This function retrieves a frequency characteristic for the current sound
  int q[10]; //Set of frequency characteristics
  int sum; //multiuse variable in summation & averaging processes
  for(int j=0;j<10;j++){//iterating through q[]
    if(digitalRead(pinBut)==HIGH){//jump to butPressed() if button is pressed - intended to improve response time
      butPressed();
    }
    q[j]=0; //clear any previous value of freq. characteristic
  for(int i=0;i<256;i++){//fill list of microphone inputs
    int k = analogRead(pinVol);
    fht_input[i]=k;
  }
  for(int i=1;i<256;i++){//knowing zero point of microphone signal at 534, find number of times mic signal crosses zero
    if((fht_input[i]-534)/(fht_input[i-1]-534)<0){//check if two consecutive inputs have opposite sign
      q[j]=q[j]+1; //store number of zero crossings in q[j] - this is a frequency characteristic
    }
  }
  }
  sum=0; //clear any previous value of sum
  for(int i=0;i<10;i++){//iterate through [q]
    sum=sum+q[i]; //sum all frequency characteristics from q[]
  }
  int avg_h=sum/10; //average freq. characteristics from q[]
  return avg_h; //return freq. characteristic
}

void butPressed() {//what to do if the button is pressed
  delay(500); //wait to check if long press or short press
    if(digitalRead(pinBut)==HIGH){//if long press
      record(); //record alarm signal
    }
    else{//if short press
      servo.write(10); //servo lever down
      delay(500);
      servo.write(180); //servo lever up
      delay(500);
    }
}
